#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>

#include "spawn.h"

int spawnl(int mode, const char *path, const char *argv0, const char *argv1, const char *argv2)
{
   int status  = -1;
   int err     = -1;
   int retstat = -1;
   int rd      = -1;
   int pfd[2]  = { -1, -1 };

   pid_t pid = -1;
   pid_t w   = -1;

   void(*istat)(), (*qstat)();

   fflush(stdout);
	fflush(stderr);

	if (pipe(pfd) == -1) {
#ifdef _DEBUG
	    perror("pipe");
#endif
	    return -1;
	}

	fcntl(pfd[0], F_SETFD, 1);
	fcntl(pfd[1], F_SETFD, 1);

 	istat = signal(SIGINT, SIG_IGN);
	qstat = signal(SIGQUIT, SIG_IGN);

   pid = fork();

   if (pid == -1) {
#ifdef _DEBUG
	  perror("fork");
#endif
	}
	else if (pid == 0) {
  
	  execl(path, argv0, argv1, argv2, NULL);

     err = errno;
#ifdef _DEBUG
	  perror(path);
#endif
	  write(pfd[1], (char *)&err, sizeof(err));
	  _exit(127);
	}
	else {
      close(pfd[1]);
	   rd = read(pfd[0], &err, sizeof(err));
	   close(pfd[0]);

	   while ((w = wait(&status)) != pid)
	      if (w < 0 && errno != EINTR)
		      break;

	   if (rd > 0) {
	      /* exec failed - errno was read from pipe */
	      retstat = -1;
	   } else if (w < 0) {
	      /* something went wrong in wait */
	      err = errno;
	      retstat = -1;
	   } else if (!WIFEXITED(status)) {
	      /* program probably was killed or signalled */
	      err = EINTR;
	      retstat = -1;
	   } else {
	      /* program terminated */
	      err = 0;
	      retstat = WEXITSTATUS(status);
	   }
   }

	(void) signal(SIGINT, istat);
	(void) signal(SIGQUIT, qstat);


   return retstat;
}